#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Repeater.h>
#include <Mowitz/Mowitz.h>
#include "MwHtml.h"


#define MENUBAR (1)
#define NAVBAR (2)
#define PERSBAR (4)
#define COMPBAR (8)

static int bars = MENUBAR;

static Widget topLevel, topbox;
static Widget html;
static Widget menubar, menubox, menubutton, menu;
static Widget navbar, navframe, navbox;
static Widget persbar, persframe, persbox, loc_label, loc_text;
static Widget statbar, label1, label2, compbar, compframe, compbox;
static Widget tooltip;
static Widget viewport, vscroll, hscroll, tab, tabl, tabr;

static String fallback[] = {
#include "kylie-ad.h"
	NULL
};

static struct hist {
	char *url;
	int top_row;
	struct hist *next;
} *backhist, *forwhist;

static char *history[10];

/* General functions */

static void fwfree(struct hist *h)
{
	if (h) {
		fwfree(h->next);
		MwFree(h->url);
		MwFree(h);
	}
}

static void adjust_scrollbars(Widget w)
{
	int tr, tc, tw, th;
	Dimension ww, wh;
	float pos, size;

	XtVaGetValues(html,
		XtNtopRow, &tr,
		XtNtopCol, &tc,
		XtNtotalWidth, &tw,
		XtNtotalHeight, &th,
		XtNwidth, &ww,
		XtNheight, &wh,
		(char *)0);
	pos = (float)tr/(float)th;
	size = (float)wh/(float)th;
	XawScrollbarSetThumb(vscroll, pos, size);
	pos = (float)tc/(float)tw;
	size = (float)ww/(float)tw;
	XawScrollbarSetThumb(hscroll, pos, size);
}

static void new_url(char *u)
{
	struct hist *h = MwMalloc(sizeof *h);
	char *v;
	int r;
	XtVaGetValues(html,
		XtNurl, &v,
		XtNtopRow, &r,
		(char *)0);
	if (v) v = MwStrdup(v);
	else v = MwStrdup(u);
	XtVaSetValues(html,
		XtNurl, u,
		XtNtopRow, 0,
		(char *)0);
	XtVaGetValues(html, XtNurl, &u, (char *)0);
	if (strcmp(u, v)) {
		h->url = v;
		h->top_row = r;
		h->next = backhist;
		backhist = h;
		fwfree(forwhist);
		forwhist = NULL;
	} else {
		MwFree(v);
		MwFree(h);
	}
	adjust_scrollbars(html);
}

static void k_open(void)
{
	static char path[1024];
	char name[1024] = "";
	char *patterns[] = { "All files (*)", NULL };
	char fmt[1024] = "";
	char *extra = NULL;
	int n, ext = 0;
	char b[1024];

	if (path[0] == '\0') strcpy(path, ".");
	n = MwFileselInput(topLevel, path, name, patterns, fmt, extra, ext);
	if (n) {
		MwSnprintf(b, sizeof b, "%s/%s", path, name);
		new_url(b);
	}
}

static void k_save(void)
{
	static char path[1024];
	char name[1024] = "";
	char *patterns[] = { "All files (*)", NULL };
	char fmt[1024] = "";
	char *extra = NULL;
	int n, ext = 0;
	char b[1024];
	char *url;

	if (path[0] == '\0') strcpy(path, ".");
	n = MwFileselInput(topLevel, path, name, patterns, fmt, extra, ext);
	XtVaGetValues(html, XtNurl, &url, (char *)0);
	if (n) {
		MwSnprintf(b, sizeof b, "%s/%s", path, name);
		MwHtmlSave(url, b);
	}
}

static void k_quit(void)
{
	MwMallocExit();
	exit(0);
}

static void k_close(void)
{
	k_quit();
}

static void k_new(void)
{
	MwSpawn("kylie");
}

static void k_find(void)
{
}

static void k_find_again(void)
{
}

static void k_print(void)
{
}

static void scroll_updown(float amount)
{
	Dimension height;
	int top_row, th;

	XtVaGetValues(html,
		XtNtotalHeight, &th,
		XtNheight, &height,
		XtNtopRow, &top_row,
		(char *)0);
	top_row += amount*height;
	if (top_row > th) top_row = th;
	if (top_row < 0) top_row = 0;
	XtVaSetValues(html,
		XtNtopRow, top_row,
		(char *)0);
	adjust_scrollbars(html);
}

static void k_page_down(void)
{
	scroll_updown(0.9);
}

static void k_page_up(void)
{
	scroll_updown(-0.9);
}

static void k_scroll_down(void)
{
	scroll_updown(0.1);
}

static void k_scroll_up(void)
{
	scroll_updown(-0.1);
}

static void scroll_leftright(float amount)
{
	Dimension width;
	int top_col, pw;

	XtVaGetValues(html,
		XtNtotalWidth, &pw,
		XtNwidth, &width,
		XtNtopCol, &top_col,
		(char *)0);
	top_col += amount*width;
	if (top_col > pw) top_col = pw;
	if (top_col < 0) top_col = 0;
	XtVaSetValues(html,
		XtNtopCol, top_col,
		(char *)0);
	adjust_scrollbars(html);
}

static void k_page_left(void)
{
	scroll_leftright(-0.9);
}

static void k_page_right(void)
{
	scroll_leftright(0.9);
}

static void k_scroll_left(void)
{
	scroll_leftright(-0.1);
}

static void k_scroll_right(void)
{
	scroll_leftright(0.1);
}

static void k_help(void)
{
}

static void k_error(void)
{
	MwErrorBox(topLevel, "Nothing to see here.");
}

static void k_home(void)
{
	new_url("about:kylie");
}

static void k_back(void)
{
	struct hist *h = backhist;
	char *u;
	int r;
	char *v;
	if (h) {
		XtVaGetValues(html,
			XtNurl, &v,
			XtNtopRow, &r,
			(char *)0);
		v = MwStrdup(v);
		XtVaSetValues(html,
			XtNurl, h->url,
			XtNtopRow, h->top_row,
			(char *)0);
		XtVaGetValues(html, XtNurl, &u, (char *)0);
		MwFree(h->url);
		if (strcmp(u, v)) {
			backhist = h->next;
			h->url = v;
			h->top_row = r;
			h->next = forwhist;
			forwhist = h;
		} else {
			MwFree(h);
			MwFree(v);
		}
		adjust_scrollbars(html);
	}
}

static void k_forward(void)
{
	struct hist *h = forwhist;
	char *u;
	int r;
	char *v;
	if (h) {
		XtVaGetValues(html,
			XtNurl, &v,
			XtNtopRow, &r,
			(char *)0);
		v = MwStrdup(v);
		XtVaSetValues(html,
			XtNurl, h->url,
			XtNtopRow, h->top_row,
			(char *)0);
		XtVaGetValues(html, XtNurl, &u, (char *)0);
		MwFree(h->url);
		if (strcmp(u, v)) {
			forwhist = h->next;
			h->url = v;
			h->top_row = r;
			h->next = backhist;
			backhist = h;
		} else {
			MwFree(v);
			MwFree(h);
		}
		adjust_scrollbars(html);
	}
}


/* Callbacks */

static void cb_new(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_new();
}

static void cb_print(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_print();
}

static void cb_close(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_close();
}

static void cb_find(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_find();
}

static void cb_find_again(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_find_again();
}

static void cb_prefs(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_error();
}

static void cb_back(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_back();
}

static void cb_forward(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_forward();
}

static void cb_reload(Widget w, XtPointer client_data, XtPointer call_data)
{
	char *u;
	XtVaGetValues(html, XtNurl, &u, (char *)0);
	u = MwStrdup(u);
	XtVaSetValues(html, XtNurl, u, (char *)0);
	MwFree(u);
	adjust_scrollbars(html);
}

static void cb_cancel(Widget w, XtPointer client_data, XtPointer call_data)
{
printf("cb_cancel()\n");
}

static void cb_home(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_home();
}

static void cb_quit(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_quit();
}

static void cb_error(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_error();
}

static void cb_about(Widget w, XtPointer client_data, XtPointer call_data)
{
printf("cb_about(%s)\n", XtName(w));
	MwAboutSiag(topLevel);
}

static void cb_click(Widget w, XtPointer client_data, XtPointer call_data)
{
	new_url((char *)call_data);
}

static void cb_open(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_open();
}

static void cb_save(Widget w, XtPointer client_data, XtPointer call_data)
{
	k_save();
}

static void cb_url(Widget w, XtPointer client_data, XtPointer call_data)
{
	char *u = (char *)call_data;
	MwComboTextChange(loc_text, u);
	adjust_scrollbars(html);
}

static void cb_loc(Widget w, XtPointer client_data, XtPointer call_data)
{
	int i;
	char *u, *v;
	cb_click(w, client_data, call_data);
	XtVaGetValues(html, XtNurl, &u, (char *)0);
	for (i = 0; i < 9; i++) {
		v = history[i+1];
		history[i+1] = history[i];
		MwFree(history[i]);
	}
	history[0] = MwStrdup(u);
	XtVaSetValues(loc_text,
		XtNcomboData, history,
		XtNcomboNData, 10,
		(char *)0);
}

static void cb_vscroll_jump(Widget w, XtPointer client_data, XtPointer call_data)
{
	float top;
	int top_row, th;

	XtVaGetValues(html, XtNtotalHeight, &th, (char *)0);
	XtVaGetValues(w, XtNtopOfThumb, &top, (char *)0);
	top_row = top*th;
	XtVaSetValues(html, XtNtopRow, top_row, (char *)0);
}

static void cb_vscroll_scroll(Widget w, XtPointer client_data, XtPointer call_data)
{
	int i = (long)call_data;
	Dimension length, height;
	int top_row, th;
	float amount;

	XtVaGetValues(w, XtNlength, &length, (char *)0);
	XtVaGetValues(html,
		XtNheight, &height,
		XtNtopRow, &top_row,
		XtNtotalHeight, &th,
		(char *)0);
	if (i < 0) {
		if ((length / -i) > 15) {
			amount = -0.1;
		} else {
			amount = -0.9;
		}
	} else {
		if ((length / i) > 15) {
			amount = 0.1;
		} else {
			amount = 0.9;
		}
	}
	scroll_updown(amount);
}

static void cb_hscroll_jump(Widget w, XtPointer client_data, XtPointer call_data)
{
        float top;
        int top_col, pw;

	XtVaGetValues(html, XtNtotalWidth, &pw, (char *)0);
        XtVaGetValues(w, XtNtopOfThumb, &top, (char *)0);
        top_col = top*pw;
        XtVaSetValues(html,
                XtNtopCol, top_col,
                (char *)0);
}

static void cb_hscroll_scroll(Widget w, XtPointer client_data, XtPointer call_data)
{
        int i = (long) call_data;
        Dimension length, width;
        int top_col, pw;
	float amount;

	XtVaGetValues(html, XtNtotalWidth, &pw, (char *)0);
        XtVaGetValues(w, XtNlength, &length, (char *)0);
        XtVaGetValues(html,
                        XtNwidth, &width,
                        XtNtopCol, &top_col,
                        (char *)0);
        if (i < 0) {
                if ((length / -i) > 15) {
			amount = -0.1;
                } else {
			amount = -0.9;
                }
        } else {
                if ((length / i) > 15) {
			amount = 0.1;
                } else {
			amount = 0.9;
                }
        }
	scroll_leftright(amount);
}


/* Actions */

static void ac_open(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_open();
}

static void ac_save(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_save();
}

static void ac_close(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_close();
}

static void ac_new(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_new();
}

static void ac_find(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_find();
}

static void ac_find_again(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_find_again();
}

static void ac_print(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_print();
}

static void ac_page_down(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_page_down();
}

static void ac_page_up(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_page_up();
}

static void ac_scroll_down(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_scroll_down();
}

static void ac_scroll_up(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_scroll_up();
}

static void ac_scroll_left(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_scroll_left();
}

static void ac_scroll_right(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_scroll_right();
}

static void ac_help(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_help();
}

static void ac_dumphistory(Widget w, XEvent *event, String *params, Cardinal *n)
{
	struct hist *h;

	printf("Backward:");
	for (h = backhist; h; h = h->next) {
		printf(" %s", h->url);
	}
	printf("\n");
	printf("Forward:");
	for (h = forwhist; h; h = h->next) {
		printf("%s", h->url);
	}
	printf("\n");
}

static void ac_quit(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_quit();
}

static void ac_back(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_back();
}

static void ac_forward(Widget w, XEvent *event, String *params, Cardinal *n)
{
	k_forward();
}


static XtActionsRec actions[] =
{
	{"open", ac_open},
	{"save", ac_save},
	{"close", ac_close},
	{"new", ac_new},
	{"find", ac_find},
	{"find_again", ac_find_again},
	{"print", ac_print},
	{"dumphistory", ac_dumphistory},
	{"quit", ac_quit},
	{"page_down", ac_page_down},
	{"page_up", ac_page_up},
	{"scroll_down", ac_scroll_down},
	{"scroll_up", ac_scroll_up},
	{"scroll_left", ac_scroll_left},
	{"scroll_right", ac_scroll_right},
	{"help", ac_help},
	{"back", ac_back},
	{"forward", ac_forward},
};


static void remake_ylayout(void)
{
	char b[100];
	sprintf(b, "%s %s %s 100%% 30",
		(bars&MENUBAR)?"30":"0",
		(bars&NAVBAR)?"30":"0",
		(bars&PERSBAR)?"30":"0");
	XtVaSetValues(topbox,
		XtNyLayout, b,
		(char *)0);
	sprintf(b, "4 50 4 100%% 4 %s",
		(bars&COMPBAR)?"175 4":"0 0");
	XtVaSetValues(statbar,
		XtNxLayout, b,
		(char *)0);
}

static void attach(Widget w, XtPointer client_data, XtPointer call_data)
{
	Widget vw = (Widget)call_data;
	if (vw == menubar) bars |= MENUBAR;
	else if (vw == navbar) bars |= NAVBAR;
	else if (vw == persbar) bars |= PERSBAR;
	else if (vw == compbar) bars |= COMPBAR;
	remake_ylayout();
}

static void detach(Widget w, XtPointer client_data, XtPointer call_data)
{
	Widget vw = (Widget)call_data;
	if (vw == menubar) bars &= ~MENUBAR;
	else if (vw == navbar) bars &= ~NAVBAR;
	else if (vw == persbar) bars &= ~PERSBAR;
	else if (vw == compbar) bars &= ~COMPBAR;
	remake_ylayout();
}

static Widget add_command(Widget pw, void (*cb)(Widget, XtPointer, XtPointer),
		XtPointer closure, char *pm, char *t)
{
	Widget w;
	Pixmap pm_return;
	Pixel color;
	XtVaGetValues(pw, XtNbackground, &color, (char *)0);
	w = XtVaCreateManagedWidget("tb_cmd",
		commandWidgetClass, pw,
		XtNforeground, color,
		(char *)0);
	pm_return = MwLoadPixmap(XtDisplay(pw), color, pm);
	XtVaSetValues(w, XtNbitmap, pm_return, (char *)0);
	XtAddCallback(w, XtNcallback, cb, closure);
	MwTooltipAdd(tooltip, w, _(t));
	return w;
}

static Widget add_menu_entry(Widget menu, void (*cb)(Widget, XtPointer, XtPointer),
		XtPointer closure, char *t)
{
	Widget w = XtVaCreateManagedWidget(t,
		mwLabelMEObjectClass, menu,
		XtNlabel, _(t),
		(char *)0);
	XtAddCallback(w, XtNcallback, cb, closure);
	return w;
}

static void add_menu_line(Widget menu)
{
	XtVaCreateManagedWidget("-",
		mwLineMEObjectClass, menu,
		(char *)0);
}

int main(int argc, char **argv)
{
	Atom wm_protocols, wm_delete_window;
	XtAppContext ac;
	char b[1024], *p;
	int i;

	if (argc == 3) {
		extern char *x_resolve_url(char *, char *);
		char *r = x_resolve_url(argv[1], argv[2]);
		printf("'%s' + '%s' = '%s'\n", argv[1], argv[2], r);
		return 0;
	}

	MwMallocInit(0, 0);

	p = getenv("HOME");
	if (!p) p = "/tmp";
	sprintf(b, "%s/.kylie", p);
	mkdir(b, 0700);
	strcat(b, "/cache");
	mkdir(b, 0700);

	if (getenv("PIXPATH") == NULL) {
		sprintf(b, "PIXPATH=%s", DEFAULT_DATAPATH);
		putenv(b);
	}
	topLevel = XtVaAppInitialize(&ac, "Kylie",
		NULL, 0,
		&argc, argv,
		fallback,
		XtNwidth, 300,
		XtNheight, 200,
		(char *)0);
	XtAppAddActions(ac, actions, XtNumber(actions));
	tooltip = XtVaCreatePopupShell("tooltip",
		mwTooltipWidgetClass, topLevel,
		(char *)0);
	MwHighlightInit(topLevel);
	topbox = XtVaCreateManagedWidget("topbox",
		mwRudegridWidgetClass, topLevel,
		(char *)0);
	menubar = XtVaCreateManagedWidget("menubar",
		mwRudegridWidgetClass, topbox,
		(char *)0);
	MwMakeHandle(menubar, menubar, detach, attach);
	menubox = XtVaCreateManagedWidget("menubox",
		mwMenuBarWidgetClass, menubar,
		(char *)0);
	bars |= MENUBAR;
	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNlabel, _("File"),
		XtNmenu_name, "filemenu",
		(char *)0);
	menu = XtVaCreatePopupShell("filemenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	add_menu_entry(menu, cb_new, NULL, "New");
	add_menu_entry(menu, cb_open, NULL, "Open Page...");
	add_menu_entry(menu, cb_save, NULL, "Save As...");
	add_menu_line(menu);
	add_menu_entry(menu, cb_error, NULL, "Send Page");
	add_menu_entry(menu, cb_error, NULL, "Send Link");
	add_menu_line(menu);
	add_menu_entry(menu, cb_error, NULL, "Edit Page");
	add_menu_line(menu);
	add_menu_entry(menu, cb_print, NULL, "Print...");
	add_menu_line(menu);
	add_menu_entry(menu, cb_close, NULL, "Close");
	add_menu_entry(menu, cb_quit, NULL, "Exit");

	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNlabel, _("Edit"),
		XtNmenu_name, "editmenu",
		(char *)0);
	menu = XtVaCreatePopupShell("editmenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	add_menu_entry(menu, cb_find, NULL, "Find in Page...");
	add_menu_entry(menu, cb_find_again, NULL, "Find Again");
	add_menu_line(menu);
	add_menu_entry(menu, cb_prefs, NULL, "Preferences...");

	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNlabel, _("View"),
		XtNmenu_name, "viewmenu",
		(char *)0);
	menu = XtVaCreatePopupShell("viewmenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	add_menu_entry(menu, cb_reload, NULL, "Reload");

	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNlabel, _("Go"),
		XtNmenu_name, "gomenu",
		(char *)0);
	menu = XtVaCreatePopupShell("gomenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	add_menu_entry(menu, cb_back, NULL, "Back");
	add_menu_entry(menu, cb_forward, NULL, "Forward");
	add_menu_entry(menu, cb_home, NULL, "Home");

	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNgravitation, XtCright,
		XtNlabel, _("Help"),
		XtNmenu_name, "helpmenu",
		(char *)0);
	menu = XtVaCreatePopupShell("helpmenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	add_menu_entry(menu, cb_error, NULL, "Contents");
	add_menu_line(menu);
	add_menu_entry(menu, cb_about, NULL, "About");

	navbar = XtVaCreateManagedWidget("navbar",
		mwRudegridWidgetClass, topbox,
		(char *)0);
	bars |= NAVBAR;
	MwMakeHandle(navbar, navbar, detach, attach);
	navframe = XtVaCreateManagedWidget("navframe",
		mwFrameWidgetClass, navbar,
		(char *)0);
	navbox = XtVaCreateManagedWidget("navbox",
		boxWidgetClass, navframe,
		(char *)0);
	add_command(navbox, cb_back, NULL, "back.xpm", "Back");
	add_command(navbox, cb_forward, NULL, "forward.xpm", "Forward");
	add_command(navbox, cb_reload, NULL, "reload.xpm", "Reload");
	add_command(navbox, cb_cancel, NULL, "cancel.xpm", "Cancel");
	add_command(navbox, cb_home, NULL, "home.xpm", "Home");
	add_command(navbox, cb_open, NULL, "fld_open.xpm", "Open");
	add_command(navbox, cb_save, NULL, "save.xpm", "Save");

	persbar = XtVaCreateManagedWidget("persbar",
		mwRudegridWidgetClass, topbox, (char *)0);
	bars |= PERSBAR;
	MwMakeHandle(persbar, persbar, detach, attach);
	persframe = XtVaCreateManagedWidget("persframe",
		mwFrameWidgetClass, persbar,
		(char *)0);
	persbox = XtVaCreateManagedWidget("persbox",
		mwRudegridWidgetClass, persframe,
		(char *)0);
	loc_label = XtVaCreateManagedWidget("loc_label",
		labelWidgetClass, persbox,
		XtNlabel, _("Location:"),
		(char *)0);
	loc_text = XtVaCreateManagedWidget("loc_text",
		mwComboWidgetClass, persbox,
		(char *)0);
	for (i = 0; i < 10; i++) {
		history[i] = MwStrdup("");
	}
	XtVaSetValues(loc_text,
		XtNcomboData, history,
		XtNcomboNData, 10,
		(char *)0);
	XtVaSetValues(loc_text, "The textfield is borken");
	XtAddCallback(loc_text, XtNtextCallback, cb_loc, NULL);
	XtAddCallback(loc_text, XtNlistCallback, cb_loc, NULL);

	statbar = XtVaCreateManagedWidget("statbar",
		mwRudegridWidgetClass, topbox, (char *)0);
	label1 = XtVaCreateManagedWidget("label1",
		labelWidgetClass, statbar,
		(char *)0);
	label2 = XtVaCreateManagedWidget("label2",
		labelWidgetClass, statbar,
		(char *)0);

	compbar = XtVaCreateManagedWidget("compbar",
		mwRudegridWidgetClass, statbar, (char *)0);
	bars |= COMPBAR;
	MwMakeHandle(compbar, compbar, detach, attach);
	compframe = XtVaCreateManagedWidget("compframe",
		mwFrameWidgetClass, compbar,
		(char *)0);
	compbox = XtVaCreateManagedWidget("compbox",
		boxWidgetClass, compframe,
		(char *)0);
	add_command(compbox, cb_error, NULL, "navigator.xpm", "Navigator");
	add_command(compbox, cb_error, NULL, "mail.xpm", "Mail");
	add_command(compbox, cb_error, NULL, "news.xpm", "News");
	add_command(compbox, cb_error, NULL, "addressbook.xpm", "Address book");
	add_command(compbox, cb_error, NULL, "composer.xpm", "Editor");

	viewport = XtVaCreateManagedWidget("viewport",
		mwRudegridWidgetClass, topbox,
		(char *)0);

	html = XtVaCreateManagedWidget("html",
		mwHtmlWidgetClass, viewport,
		XtNstatus, label2,
		(char *)0);
	XtAddCallback(html, XtNcallback, cb_click, NULL);
	XtAddCallback(html, XtNchangeUrl, cb_url, NULL);
	if (argc < 2) {
		XtVaSetValues(html, XtNurl, "about:kylie", (char *)0);
	} else {
		XtVaSetValues(html, XtNurl, argv[1], (char *)0);
	}
	vscroll = XtVaCreateManagedWidget("vscroll",
		scrollbarWidgetClass, viewport,
		(char *)0);
	XtAddCallback(vscroll, XtNjumpProc, cb_vscroll_jump, NULL);
	XtAddCallback(vscroll, XtNscrollProc, cb_vscroll_scroll, NULL);
	hscroll = XtVaCreateManagedWidget("hscroll",
		scrollbarWidgetClass, viewport,
		(char *)0);
	XtAddCallback(hscroll, XtNjumpProc, cb_hscroll_jump, NULL);
	XtAddCallback(hscroll, XtNscrollProc, cb_hscroll_scroll, NULL);
	tab = XtVaCreateManagedWidget("tab",
		mwTabbingWidgetClass, viewport,
		(char *)0);
	MwTabbingInsert(tab, "Page 1", -1);
	MwTabbingInsert(tab, "Page 2", -1);
	tabl = XtVaCreateManagedWidget("tabl",
		repeaterWidgetClass, viewport,
		(char *)0);
	tabr = XtVaCreateManagedWidget("tabr",
		repeaterWidgetClass, viewport,
		(char *)0);

	tooltip = XtVaCreatePopupShell("tooltip",
		mwTooltipWidgetClass, topLevel,
		(char *)0);
	XtRealizeWidget(topLevel);
	wm_protocols = XInternAtom(XtDisplay(topLevel),
				"WM_PROTOCOLS", False);
	wm_delete_window = XInternAtom(XtDisplay(topLevel),
				"WM_DELETE_WINDOW", False);
	XtOverrideTranslations(topLevel,
		XtParseTranslationTable(
			"<Message>WM_PROTOCOLS: quit()"));
	XSetWMProtocols(XtDisplay(topLevel), XtWindow(topLevel),
			&wm_delete_window, 1);
	XtAppMainLoop(ac);
	return 0;
}
